// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024-2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aoc/LanParty.hpp>

const std::string LanParty::getPuzzleName() const
{
    return "LAN Party";
}

const int LanParty::getPuzzleDay() const
{
    return 23;
}

void LanParty::processDataLine(const std::string& line)
{
    lan_.addEdge(line.substr(0, 2), line.substr(3, 2));
}

void LanParty::finish()
{
    auto vertices = lan_.getVertices();
    // This works as long as "ta" is present as a computername, otherwise we would have to use vertices.find_if().
    auto iterTx = vertices.find(getFirstTxComputerName());
    while (iterTx != vertices.end() && iterTx->second->getId() <= getLastTxComputerName())
    {
        computeInterconnectedThreeSetCount(*iterTx->second);
        iterTx++;
    }
}

void LanParty::computeInterconnectedThreeSetCount(const Vertex& vertexTx)
{
    auto neighborsOfTx = vertexTx.getNeighbors();
    for (size_t i = 0; i < neighborsOfTx.size(); i++)
    {
        auto vertexA = neighborsOfTx[i].lock();
        if (shallProcessVertex(*vertexA, vertexTx))
        {
            for (size_t j{ i + 1 }; j < neighborsOfTx.size(); j++)
            {
                auto vertexB = neighborsOfTx[j].lock();
                if (shallProcessVertex(*vertexB, vertexTx))
                {
                    for (auto& neighborOfA : vertexA->getNeighbors())
                    {
                        if (vertexB == neighborOfA.lock())
                        {
                            part1++;
                        }
                    }
                }
            }
        }
    }
}

bool LanParty::shallProcessVertex(const Vertex& vertexToCheck, const Vertex& vertexTx) const
{
    return (vertexToCheck.getId()[0] != 't' || vertexToCheck.getId() > vertexTx.getId());
}

const std::string LanParty::getFirstTxComputerName() const
{
    return "ta";
}

const std::string LanParty::getLastTxComputerName() const
{
    return "tz";
}
