// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024-2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include <aocTests/TestContext.hpp>

#include <catch2/catch_test_macros.hpp>

#include <aoc/framework/SolverEngine.hpp>

void TestContext::run(const std::unique_ptr<Solver<long long, long long>>&& solver, const long long expected1,
    const long long expected2, const std::vector<std::string>& inputPaths)
{
    SolverEngine solverEngine{ inputPaths };
    solverEngine.run(*solver);

    REQUIRE(expected1 == solver->getResultPart1());
    REQUIRE(expected2 == solver->getResultPart2());
}

void TestContext::run(const std::unique_ptr<Solver<long long, std::string>>&& solver, const long long expected1,
    const std::string expected2, const std::vector<std::string>& inputPaths)
{
    SolverEngine solverEngine{ inputPaths };
    solverEngine.run(*solver);

    REQUIRE(expected1 == solver->getResultPart1());
    REQUIRE(expected2 == solver->getResultPart2());
}

void TestContext::runPart1(const std::unique_ptr<Solver<long long, long long>>&& solver, const long long expected,
    const std::vector<std::string>& inputPaths)
{
    SolverEngine solverEngine{ inputPaths };
    solverEngine.run(*solver);

    REQUIRE(expected == solver->getResultPart1());
}

void TestContext::runPart2(const std::unique_ptr<Solver<long long, long long>>&& solver, const long long expected,
    const std::vector<std::string>& inputPaths)
{
    SolverEngine solverEngine{ inputPaths };
    solverEngine.run(*solver);

    REQUIRE(expected == solver->getResultPart2());
}

std::vector<std::string> TestContext::getInputPaths() const
{
    return std::vector<std::string>{ "data", "../../../data", "../../../../data" };
}

std::vector<std::string> TestContext::getExampleInputPaths() const
{
    return std::vector<std::string>{ "data/examples", "../../../data/examples", "../../../../data/examples" };
}
